import { useReducer, useCallback } from 'react';

function handleFetching(resource) {
  return {
    type: resource + "/FETCHING",
    payload: {
      resource: resource
    }
  };
}

function handleSuccess(resource, response) {
  return {
    type: resource + "/SUCCESS",
    payload: {
      resource: resource,
      response: response
    }
  };
}

function handleError(resource, error) {
  return {
    type: resource + "/ERROR",
    payload: {
      resource: resource,
      error: error
    }
  };
}

function logger(reducer) {
  return function (state, action) {
    console.info("%c Previous State:", "font-weight: bold; color: #2471A3;", state);
    console.info("%c Action", "font-weight: bold; color: #CB4335;", action.type);

    var nextState = reducer(state, action);

    console.info("%c Next State", "font-weight: bold; color: #17A589;", nextState);

    return nextState;
  };
}

var defineProperty = function (obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
};

var _extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};

var objectWithoutProperties = function (obj, keys) {
  var target = {};

  for (var i in obj) {
    if (keys.indexOf(i) >= 0) continue;
    if (!Object.prototype.hasOwnProperty.call(obj, i)) continue;
    target[i] = obj[i];
  }

  return target;
};

var slicedToArray = function () {
  function sliceIterator(arr, i) {
    var _arr = [];
    var _n = true;
    var _d = false;
    var _e = undefined;

    try {
      for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
        _arr.push(_s.value);

        if (i && _arr.length === i) break;
      }
    } catch (err) {
      _d = true;
      _e = err;
    } finally {
      try {
        if (!_n && _i["return"]) _i["return"]();
      } finally {
        if (_d) throw _e;
      }
    }

    return _arr;
  }

  return function (arr, i) {
    if (Array.isArray(arr)) {
      return arr;
    } else if (Symbol.iterator in Object(arr)) {
      return sliceIterator(arr, i);
    } else {
      throw new TypeError("Invalid attempt to destructure non-iterable instance");
    }
  };
}();

var toConsumableArray = function (arr) {
  if (Array.isArray(arr)) {
    for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) arr2[i] = arr[i];

    return arr2;
  } else {
    return Array.from(arr);
  }
};

var filterFetching = function filterFetching(action, state) {
  return state.fetching.filter(function (resource) {
    return !action.payload.resource.includes(resource);
  });
};

var filterResourcesOrErrors = function filterResourcesOrErrors(resource, resources) {
  var r = Object.keys(resources).reduce(function (acc, res) {
    if (!resource.includes(res)) {
      acc[res] = resources[res];
    }

    return acc;
  }, {});

  return r;
};

function createReducer() {
  return function (state, action) {
    var resource = action.payload.resource;


    switch (action.type) {
      case resource + '/FETCHING':
        return {
          fetching: [].concat(toConsumableArray(state.fetching), [action.payload.resource]).reduce(function (acc, val) {
            return acc.concat(val);
          }, []),
          resources: filterResourcesOrErrors(action.payload.resource, state.resources),
          errors: filterResourcesOrErrors(action.payload.resource, state.errors)
        };
      case resource + '/SUCCESS':
        return {
          fetching: filterFetching(action, state),
          resources: _extends({}, state.resources, action.payload.response),
          errors: filterResourcesOrErrors(action.payload.resource, state.errors)
        };
      case resource + '/ERROR':
        return {
          fetching: filterFetching(action, state),
          resources: filterResourcesOrErrors(action.payload.resource, state.resources),
          errors: _extends({}, state.errors, action.payload.error)
        };
    }
  };
}

function buildReducer(debug) {
  var reducer = createReducer();

  /* istanbul ignore next */
  if (debug === true && process.env.NODE_ENV === 'development') {
    return logger(reducer);
  } else {
    return reducer;
  }
}

function useApiRequest(_ref) {
  var axios = _ref.axios,
      key = _ref.key,
      debug = _ref.debug;

  var initialState = {
    fetching: [],
    resources: {},
    errors: {}
  };

  var reducer = useCallback(buildReducer(debug), [key]);

  var _useReducer = useReducer(reducer, initialState),
      _useReducer2 = slicedToArray(_useReducer, 2),
      state = _useReducer2[0],
      dispatch = _useReducer2[1];

  var replaceParams = function replaceParams(url, response) {
    var matches = url.match(/\{{.*?\}}/g);

    if (matches && matches.length > 0) {
      var params = matches.map(function (match) {
        return match.replace("{{", "").replace("}}", "");
      });

      return matches.reduce(function (acc, match, i) {
        var param = params[i].split(".").reduce(function (pAcc, fragment, j) {
          return pAcc[fragment];
        }, response);

        return acc.replace(match, param);
      }, url);
    } else {
      return url;
    }
  };

  var onError = function onError(resource, error) {
    dispatch(handleError(resource, error));
  };

  var makeApiRequest = function makeApiRequest(config, prevResponse) {
    var resource = Object.keys(config)[0];
    var _config$resource = config[resource],
        next = _config$resource.next,
        isNext = _config$resource.isNext,
        url = _config$resource.url,
        rest = objectWithoutProperties(_config$resource, ["next", "isNext", "url"]);

    var request = _extends({}, rest, { url: replaceParams(url, prevResponse) });

    if (!isNext) dispatch(handleFetching([resource]));

    axios.request(request).then(function (response) {
      if (next) {
        var nextConfig = defineProperty({}, resource, _extends({}, next, { isNext: true }));

        makeApiRequest(nextConfig, response);
      } else {
        dispatch(handleSuccess([resource], defineProperty({}, resource, response)));
      }
    }).catch(function (e) {
      return onError([resource], defineProperty({}, resource, e));
    });
  };

  var makeApiRequests = function makeApiRequests(config) {
    var resources = Object.keys(config);

    dispatch(handleFetching(resources));

    Promise.all(resources.map(function (key) {
      return axios.request(_extends({}, config[key], {
        headers: {
          resourceKey: key
        }
      }));
    })).then(function (responses) {
      var response = resources.reduce(function (acc, resource, i) {
        acc[resource] = responses[i];

        return acc;
      }, {});

      dispatch(handleSuccess(resources, response));
    }).catch(function (e) {
      var resource = e.config.headers.resourceKey;

      return onError(resources, defineProperty({}, resource, e));
    });
  };

  return { state: state, makeApiRequest: makeApiRequest, makeApiRequests: makeApiRequests };
}

export { useApiRequest };
//# sourceMappingURL=index.es.js.map
